@@ -1,13 +1,14 @@
-// Atomic{I,U}128 implementation for aarch64.
+// Atomic{I,U}128 implementation for AArch64.
 //
-// There are two ways to implement 128-bit atomic operations in aarch64.
+// There are two ways to implement 128-bit atomic operations in AArch64.
 //
 // - LDXP/STXP loop (DW LL/SC)
 // - CASP (DWCAS) added as FEAT_LSE
 //
-// Currently, we always use CASP if FEAT_LSE is enabled at compile-time,
-// otherwise, always use LDXP/STXP loop. In the future, we may want to support
-// outline atomics or mix of both methods.
+// If the `outline-atomics` feature is not enabled, we always use CASP if
+// FEAT_LSE is enabled at compile-time, otherwise, always use LDXP/STXP loop.
+// If the `outline-atomics` feature is enabled, we use CASP for
+// compare_exchange(_weak) if FEAT_LSE is available at run-time.
 //
 // Refs:
 // - ARM Compiler armasm User Guide
@@ -15,6 +16,7 @@
 // - Arm Architecture Reference Manual for A-profile architecture
 //   https://developer.arm.com/documentation/ddi0487/latest
 // - progress64 https://github.com/ARM-software/progress64
+// - atomic-maybe-uninit https://github.com/taiki-e/atomic-maybe-uninit
 //
 // Generated asm(default): https://godbolt.org/z/4nvGW91Mh
 // Generated asm(+lse): https://godbolt.org/z/s7Tc59odP
@@ -47,7 +49,7 @@ union U128 {
 unsafe fn _ldxp(src: *mut u128, order: Ordering) -> u128 {
     debug_assert!(src as usize % 16 == 0);
 
-    // SAFETY: the caller must guarantee that `dst` is valid for both writes and
+    // SAFETY: the caller must guarantee that `src` is valid for both writes and
     // reads, 16-byte aligned, and that there are no concurrent non-atomic operations.
     //
     // Refs:
@@ -60,8 +62,8 @@ unsafe fn _ldxp(src: *mut u128, order: Ordering) -> u128 {
                 asm!(
                     "ldxp {prev_lo}, {prev_hi}, [{src}]",
                     src = in(reg) src,
-                    prev_lo = lateout(reg) prev_lo,
-                    prev_hi = lateout(reg) prev_hi,
+                    prev_lo = out(reg) prev_lo,
+                    prev_hi = out(reg) prev_hi,
                     options(nostack),
                 );
             }
@@ -69,8 +71,8 @@ unsafe fn _ldxp(src: *mut u128, order: Ordering) -> u128 {
                 asm!(
                     "ldaxp {prev_lo}, {prev_hi}, [{src}]",
                     src = in(reg) src,
-                    prev_lo = lateout(reg) prev_lo,
-                    prev_hi = lateout(reg) prev_hi,
+                    prev_lo = out(reg) prev_lo,
+                    prev_hi = out(reg) prev_hi,
                     options(nostack),
                 );
             }
@@ -122,7 +124,17 @@ unsafe fn _stxp(dst: *mut u128, val: u128, order: Ordering) -> bool {
     }
 }
 
-#[cfg(any(test, portable_atomic_target_feature_lse, target_feature = "lse"))]
+#[cfg(any(
+    portable_atomic_target_feature_lse,
+    target_feature = "lse",
+    all(
+        portable_atomic_aarch64_target_feature,
+        feature = "outline-atomics",
+        // https://github.com/rust-lang/stdarch/blob/bcbe010614f398ec86f3a9274d22e33e5f2ee60b/crates/std_detect/src/detect/mod.rs
+        any(feature = "std", target_os = "linux", target_os = "windows", target_os = "freebsd")
+    ),
+))]
+#[cfg_attr(portable_atomic_aarch64_target_feature, target_feature(enable = "lse"))]
 #[inline]
 unsafe fn _casp(dst: *mut u128, old: u128, new: u128, order: Ordering) -> u128 {
     debug_assert!(dst as usize % 16 == 0);
@@ -142,9 +154,10 @@ unsafe fn _casp(dst: *mut u128, old: u128, new: u128, order: Ordering) -> u128 {
                 asm!(
                     "casp x6, x7, x4, x5, [{dst}]",
                     dst = in(reg) dst,
-                    // Note: `old` and `new` must be allocated to even/odd register pairs.
-                    inlateout("x6") old.pair[0] => prev_lo,
-                    inlateout("x7") old.pair[1] => prev_hi,
+                    // must be allocated to even/odd register pair
+                    inout("x6") old.pair[0] => prev_lo,
+                    inout("x7") old.pair[1] => prev_hi,
+                    // must be allocated to even/odd register pair
                     in("x4") new.pair[0],
                     in("x5") new.pair[1],
                     options(nostack),
@@ -154,9 +167,10 @@ unsafe fn _casp(dst: *mut u128, old: u128, new: u128, order: Ordering) -> u128 {
                 asm!(
                     "caspa x6, x7, x4, x5, [{dst}]",
                     dst = in(reg) dst,
-                    // Note: `old` and `new` must be allocated to even/odd register pairs.
-                    inlateout("x6") old.pair[0] => prev_lo,
-                    inlateout("x7") old.pair[1] => prev_hi,
+                    // must be allocated to even/odd register pair
+                    inout("x6") old.pair[0] => prev_lo,
+                    inout("x7") old.pair[1] => prev_hi,
+                    // must be allocated to even/odd register pair
                     in("x4") new.pair[0],
                     in("x5") new.pair[1],
                     options(nostack),
@@ -166,9 +180,10 @@ unsafe fn _casp(dst: *mut u128, old: u128, new: u128, order: Ordering) -> u128 {
                 asm!(
                     "caspl x6, x7, x4, x5, [{dst}]",
                     dst = in(reg) dst,
-                    // Note: `old` and `new` must be allocated to even/odd register pairs.
-                    inlateout("x6") old.pair[0] => prev_lo,
-                    inlateout("x7") old.pair[1] => prev_hi,
+                    // must be allocated to even/odd register pair
+                    inout("x6") old.pair[0] => prev_lo,
+                    inout("x7") old.pair[1] => prev_hi,
+                    // must be allocated to even/odd register pair
                     in("x4") new.pair[0],
                     in("x5") new.pair[1],
                     options(nostack),
@@ -178,9 +193,10 @@ unsafe fn _casp(dst: *mut u128, old: u128, new: u128, order: Ordering) -> u128 {
                 asm!(
                     "caspal x6, x7, x4, x5, [{dst}]",
                     dst = in(reg) dst,
-                    // Note: `old` and `new` must be allocated to even/odd register pairs.
-                    inlateout("x6") old.pair[0] => prev_lo,
-                    inlateout("x7") old.pair[1] => prev_hi,
+                    // must be allocated to even/odd register pair
+                    inout("x6") old.pair[0] => prev_lo,
+                    inout("x7") old.pair[1] => prev_hi,
+                    // must be allocated to even/odd register pair
                     in("x4") new.pair[0],
                     in("x5") new.pair[1],
                     options(nostack),
@@ -219,25 +235,68 @@ unsafe fn atomic_compare_exchange(
     success: Ordering,
     _failure: Ordering,
 ) -> Result<u128, u128> {
+    #[inline]
+    unsafe fn _compare_exchange_ldxp_stxp(
+        dst: *mut u128,
+        old: u128,
+        new: u128,
+        success: Ordering,
+    ) -> u128 {
+        // SAFETY: the caller must uphold the safety contract for `_compare_exchange_ldxp_stxp`.
+        unsafe { atomic_update(dst, success, |x| if x == old { new } else { x }) }
+    }
+
     #[cfg(any(portable_atomic_target_feature_lse, target_feature = "lse"))]
     // SAFETY: the caller must uphold the safety contract for `atomic_compare_exchange`.
-    unsafe {
-        let res = _casp(dst, old, new, success);
-        if res == old {
-            Ok(res)
-        } else {
-            Err(res)
-        }
-    }
+    let res = unsafe { _casp(dst, old, new, success) };
+    #[cfg(not(all(
+        portable_atomic_aarch64_target_feature,
+        feature = "outline-atomics",
+        // https://github.com/rust-lang/stdarch/blob/bcbe010614f398ec86f3a9274d22e33e5f2ee60b/crates/std_detect/src/detect/mod.rs
+        any(feature = "std", target_os = "linux", target_os = "windows", target_os = "freebsd")
+    )))]
     #[cfg(not(any(portable_atomic_target_feature_lse, target_feature = "lse")))]
     // SAFETY: the caller must uphold the safety contract for `atomic_compare_exchange`.
-    unsafe {
-        let res = atomic_update(dst, success, |x| if x == old { new } else { x });
-        if res == old {
-            Ok(res)
-        } else {
-            Err(res)
+    let res = unsafe { _compare_exchange_ldxp_stxp(dst, old, new, success) };
+    #[cfg(all(
+        portable_atomic_aarch64_target_feature,
+        feature = "outline-atomics",
+        // https://github.com/rust-lang/stdarch/blob/bcbe010614f398ec86f3a9274d22e33e5f2ee60b/crates/std_detect/src/detect/mod.rs
+        any(feature = "std", target_os = "linux", target_os = "windows", target_os = "freebsd")
+    ))]
+    #[cfg(not(any(portable_atomic_target_feature_lse, target_feature = "lse")))]
+    let res = {
+        extern crate std;
+        // Adapted from https://github.com/BurntSushi/memchr/blob/2.4.1/src/memchr/x86/mod.rs#L9-L71.
+        use core::{mem, sync::atomic::AtomicPtr};
+        type FnRaw = *mut ();
+        type FnTy = unsafe fn(*mut u128, u128, u128, Ordering) -> u128;
+        static FUNC: AtomicPtr<()> = AtomicPtr::new(detect as FnRaw);
+        #[cold]
+        unsafe fn detect(dst: *mut u128, old: u128, new: u128, success: Ordering) -> u128 {
+            let func: FnTy = if std::arch::is_aarch64_feature_detected!("lse") {
+                _casp
+            } else {
+                _compare_exchange_ldxp_stxp
+            };
+            FUNC.store(func as FnRaw, Ordering::Relaxed);
+            // SAFETY: the caller must guarantee that `dst` is valid for both writes and
+            // reads, 16-byte aligned, that there are no concurrent non-atomic operations,
+            // and we’ve checked if FEAT_LSE is available,
+            unsafe { func(dst, old, new, success) }
         }
+        // SAFETY: `FnTy` is a function pointer, which is always safe to transmute with a `*mut ()`.
+        // the caller must guarantee that `dst` is valid for both writes and
+        // reads, 16-byte aligned, and that there are no concurrent non-atomic operations.
+        unsafe {
+            let func = FUNC.load(Ordering::Relaxed);
+            mem::transmute::<FnRaw, FnTy>(func)(dst, old, new, success)
+        }
+    };
+    if res == old {
+        Ok(res)
+    } else {
+        Err(res)
     }
 }
 
