@@ -10,8 +10,13 @@ mod core_atomic;
 #[cfg(target_arch = "aarch64")]
 mod aarch64;
 
-#[cfg(any(test, portable_atomic_cmpxchg16b))]
+#[cfg(any(test, not(portable_atomic_core_atomic_128)))]
 #[cfg(any(not(portable_atomic_no_asm), portable_atomic_nightly))]
+#[cfg(any(
+    portable_atomic_target_feature_cmpxchg16b,
+    target_feature = "cmpxchg16b",
+    portable_atomic_cmpxchg16b_dynamic
+))]
 #[cfg(target_arch = "x86_64")]
 mod cmpxchg16b;
 
@@ -34,7 +39,12 @@ mod riscv;
     test,
     not(any(
         portable_atomic_core_atomic_128,
-        all(portable_atomic_cmpxchg16b, not(portable_atomic_cmpxchg16b_dynamic)),
+        all(
+            not(portable_atomic_core_atomic_128),
+            any(not(portable_atomic_no_asm), portable_atomic_nightly),
+            any(portable_atomic_target_feature_cmpxchg16b, target_feature = "cmpxchg16b"),
+            target_arch = "x86_64",
+        ),
         all(
             not(portable_atomic_core_atomic_128),
             any(not(portable_atomic_no_asm), portable_atomic_nightly),
@@ -190,16 +200,37 @@ pub(crate) use self::core_atomic::{AtomicI128, AtomicU128};
 ))]
 pub(crate) use self::aarch64::{AtomicI128, AtomicU128};
 // no core Atomic{I,U}128 & has cmpxchg16b => use cmpxchg16b
-#[cfg(portable_atomic_cmpxchg16b)]
+#[cfg(all(
+    not(portable_atomic_core_atomic_128),
+    any(not(portable_atomic_no_asm), portable_atomic_nightly),
+    any(
+        portable_atomic_target_feature_cmpxchg16b,
+        target_feature = "cmpxchg16b",
+        portable_atomic_cmpxchg16b_dynamic
+    ),
+    target_arch = "x86_64",
+))]
 pub(crate) use self::cmpxchg16b::{AtomicI128, AtomicU128};
 // no core Atomic{I,U}128 & has CAS => use lock-base fallback
 #[cfg(feature = "fallback")]
-#[cfg(not(all(
-    not(portable_atomic_core_atomic_128),
-    any(not(portable_atomic_no_asm), portable_atomic_nightly),
-    target_arch = "aarch64"
+#[cfg(not(any(
+    portable_atomic_core_atomic_128,
+    all(
+        not(portable_atomic_core_atomic_128),
+        any(not(portable_atomic_no_asm), portable_atomic_nightly),
+        target_arch = "aarch64"
+    ),
+    all(
+        not(portable_atomic_core_atomic_128),
+        any(not(portable_atomic_no_asm), portable_atomic_nightly),
+        any(
+            portable_atomic_target_feature_cmpxchg16b,
+            target_feature = "cmpxchg16b",
+            portable_atomic_cmpxchg16b_dynamic
+        ),
+        target_arch = "x86_64",
+    ),
 )))]
-#[cfg(not(any(portable_atomic_core_atomic_128, portable_atomic_cmpxchg16b)))]
 #[cfg_attr(
     not(portable_atomic_cfg_target_has_atomic),
     cfg(not(portable_atomic_no_atomic_cas))
