@@ -9,7 +9,7 @@ macro_rules! __test_atomic_common {
         }
         #[test]
         fn alignment() {
-            // https://github.com/rust-lang/rust/blob/1.58.1/library/core/tests/atomic.rs#L165
+            // https://github.com/rust-lang/rust/blob/1.59.0/library/core/tests/atomic.rs#L165
             assert_eq!(core::mem::align_of::<$atomic_type>(), core::mem::size_of::<$atomic_type>());
             assert_eq!(core::mem::size_of::<$atomic_type>(), core::mem::size_of::<$value_type>());
         }
@@ -918,11 +918,18 @@ macro_rules! test_atomic_ptr_pub {
     };
 }
 
+#[track_caller]
 pub(crate) fn assert_panic<T: std::fmt::Debug>(f: impl FnOnce() -> T) -> std::string::String {
+    let backtrace = std::env::var_os("RUST_BACKTRACE");
     let hook = std::panic::take_hook();
+    std::env::set_var("RUST_BACKTRACE", "0"); // Suppress backtrace
     std::panic::set_hook(std::boxed::Box::new(|_| {})); // Suppress panic msg
     let res = std::panic::catch_unwind(std::panic::AssertUnwindSafe(f));
     std::panic::set_hook(hook);
+    match backtrace {
+        Some(v) => std::env::set_var("RUST_BACKTRACE", v),
+        None => std::env::remove_var("RUST_BACKTRACE"),
+    }
     let msg = res.unwrap_err();
     msg.downcast_ref::<std::string::String>()
         .cloned()
@@ -989,7 +996,7 @@ pub(crate) fn test_store_ordering<T: std::fmt::Debug>(f: impl Fn(Ordering) -> T)
     }
 }
 pub(crate) fn compare_exchange_orderings() -> [(Ordering, Ordering); 9] {
-    // https://github.com/rust-lang/rust/blob/1.58.1/library/core/tests/atomic.rs#L199
+    // https://github.com/rust-lang/rust/blob/1.59.0/library/core/tests/atomic.rs#L199
     [
         (Ordering::Relaxed, Ordering::Relaxed),
         (Ordering::Acquire, Ordering::Relaxed),
